# coding: utf-8
import logging

from django.core.management.base import BaseCommand
from optparse import make_option

from acomea.sso.utils.match_users_dj_vs_sso import MatchUsers


class Command(BaseCommand):
    """
    Controlla che gli username degli utenti su acomea siano uguali a quelli su SSO a parità di email.
    In caso di differenza gli utenti acomea (con rac != 6 o 9) saranno offuscati perchè superati.
    In coda verrà mandato report via email
    """
    help = "Controlla e offusca gli utenti acomea (rac!>6|9) diversi da quelli SSO a parità di email."

    option_list = BaseCommand.option_list + (
        make_option('-b', '--begin', type='int', dest='begin', help='Begin from index'),
        make_option('-s', '--limit', type='int', dest='limit', help='Limit list retrieved'),
        make_option('-u', '--uid', dest='uid', help='Uid to manage', ),
        make_option('-m', '--email', dest='email', help='Email to manage', ),
        make_option('-f', '--file', dest='file', default=None, help="Import list uid from file"),

        make_option('-c', '--console', action='store_true', default=False, help='Debug - write logging to console'),
        make_option('-l', '--debug-level', default='info', help='Set debug level (debug, info, warnings) for console'),
        make_option('-x', '--no-stop', action='store_true', default=False, help="Don't stop on error, just log"),
        make_option('-n', '--dry-run', action='store_true', default=False, help="Only report, no obfuscate"),
    )

    # output_transaction = True
    logger = logging.getLogger('acomea.sso.match.obfuscate')

    def handle(self, **options):
        level = getattr(logging, options.get('debug_level').upper())
        self.logger.setLevel(level)

        if options.get('console'):
            console_handler = logging._handlers['console']
            console_handler.setLevel(level)
            self.logger.handlers = []
            self.logger.addHandler(console_handler)

        MatchUsers().do(options)

# NO BLOCK ONLY fabiopre
# dj command_match_users_dj_vs_sso -c -x --debug-level=DEBUG --uid=fabiopre
# NO BLOCK FIRST 5000 records
# dj command_match_users_dj_vs_sso -c --debug-level=DEBUG -x -s5000
# NO BLOCK BEGIN from record number 2000 for 5000 records
# dj command_match_users_dj_vs_sso -c --debug-level=DEBUG -x -b2000 -s5000
# NO BLOCK WITH INPUT FILE
# dj command_match_users_dj_vs_sso -c -x --debug-level=DEBUG  -f/tmp/e.txt
