# coding: utf-8

import logging
import os

from django.contrib.auth import models
from django.core.management.base import BaseCommand
from codecs import open
from optparse import make_option

from acomea.sso.utils.sso_client import SSOClient


class Command(BaseCommand):
    """
    Offusca utenti su acomea e aggiorna sso (se utente esiste sso)
    """
    help = "Offusca utenti su acomea e aggiorna sso (se utente esiste sso)"

    option_list = BaseCommand.option_list + (
        make_option('-b', '--begin', type='int', dest='begin', help='Begin from index'),
        make_option('-s', '--limit', type='int', dest='limit', help='Limit list retrieved'),
        make_option('-u', '--uid', dest='uid', help='Uid to manage', ),
        make_option('-f', '--file', dest='file', default=None, help="Import list uid from file"),
        make_option('-o', '--output-file', dest='output-file', default=None, help="Output file name for uid in error"),
        make_option('-d', '--delete', action='store_true', default=False, help="Delete error file"),

        make_option('-c', '--console', action='store_true', default=False, help='Debug - write logging to console'),
        make_option('-l', '--debug-level', default='info', help='Set debug level (debug, info, warnings) for console'),
        make_option('-x', '--no-stop', action='store_true', default=False, help="Don't stop on error, just log"),
    )

    output_transaction = True
    logger = logging.getLogger('acomea.sso.obfuscate')

    def handle(self, **options):

        level = getattr(logging, options.get('debug_level').upper())
        self.logger.setLevel(level)

        if options.get('console'):
            console_handler = logging._handlers['console']
            console_handler.setLevel(level)
            self.logger.handlers = []
            self.logger.addHandler(console_handler)

        begin = options.get('begin')
        limit = options.get('limit')
        uid = options.get('uid')
        file_input = options.get('file')
        obfuscate_ok = 0
        obfuscate_ko = 0
        users = models.User.objects.all()
        if uid:
            users = users.filter(username=uid)

        if file_input:
            with open(file_input, "rb") as input:
                list_to_filter = [(uid.split(";")[0]) for uid in input]
                if limit:
                    list_to_filter = list_to_filter[:limit]
                users = users.filter(username__in=list_to_filter)
        try:
            sso_client = SSOClient()

            self.logger.info('Start obfuscate users on sso')

            index_begin = 0
            if begin:
                users = users[begin:]
                index_begin += begin
            if limit:
                users = users[:limit]

            error_file_name = options.get('output-file') or "/tmp/obfuscate_acomea_to_sso.txt"

            if options.get('delete') and os.path.exists(error_file_name):
                os.remove(error_file_name)
            with open(error_file_name, "a", 'utf-8') as error_file:
                count = len(users)
                for index, user in enumerate(users):
                    self.logger.debug(u"Try {}...({})\n".format(user.username, count))
                    try:
                        sso_client.exist(username=user)
                        self.logger.info(u"{} - {} - {} --> NOT OBFUSCATED, EXIST ON SSO: {}\n".format(
                            index, (index + index_begin), count, user.username)
                        )
                        msg = u"NOT OBFUSCATED;{};\n".format(user.username)
                        error_file.write(msg)
                        obfuscate_ko += 1

                    except Exception as e:
                        try:
                            user.obfuscate()
                        except Exception:
                            self.logger.info(u"OBFUSCATED : User: {}\n".format(user.username))
                            msg = u"OBFUSCATED;{};\n".format(user.username)
                            error_file.write(msg)
                            if options.get('no_stop'):
                                pass
                            else:
                                raise
                        obfuscate_ok += 1
                    count -= 1
        except Exception as e:
            self.logger.fatal(e)
            raise
        self.logger.info(u"Obfuscated: {}\t NOT OBFUSCATED: {}".format(obfuscate_ok, obfuscate_ko))

# NO BLOCK
# dj obfuscate_user_acomea_and_sync_sso -ldebug -c -d -o/tmp/e.txt -x
# NO BLOCK
# dj obfuscate_user_acomea_and_sync_sso -ldebug -c -d -o/tmp/e.txt -ufabiopre -x
# NO BLOCK WITH INPUT FILE
# dj obfuscate_user_acomea_and_sync_sso -ldebug -c -d -f/tmp/e.txt -o/tmp/e.txt -x
# NO BLOCK WITH SIZE LIMIT
# dj obfuscate_user_acomea_and_sync_sso -ldebug -c -d -f/tmp/users_email_doppie.csv -x -s10
