# coding: utf-8
import logging
import re

from django.conf import settings
from django.contrib.auth import logout, login
from django.http import HttpResponseRedirect

from . import exceptions, models as sso_models
from .utils import sso_client


class SingleSignOn(object):
    logger = logging.getLogger('acomea.profilo.migrazione')

    def process_request(self, request):
        if re.match("%s|%s" % (settings.MEDIA_URL, settings.STATIC_URL), request.path):
            return

        if settings.ACOMEA_SSO_SERVER and hasattr(request, 'user'):
            sso_token = request.COOKIES.get('token')
            uid = request.COOKIES.get('uid')
            if sso_token and uid:
                self.logger.debug('uid:{}\ttoken:{}'.format(uid, sso_token))
                try:
                    # forzatura richiesta da cliente
                    uid = uid.lower()
                    sso_client.SSOClient().check(uid, sso_token)
                    sso_login_data = {'uid': uid, 'token': sso_token}
                    self.logger.debug('try refresh {}'.format(uid))
                    user = sso_models.refresh_or_create_dj_objects(sso_login_data=sso_login_data)
                    user.backend = 'acomea.sso.auth.SSOBackend'
                    self.logger.debug('try login {}'.format(uid))
                    login(request, user)
                    self.logger.debug('OK login {}'.format(uid))
                    return
                except exceptions.DuplicateEmailIntegrityException:
                    logout(request)
                    return HttpResponseRedirect(settings.ACOMEA_SSO_URL_REDIRECT_DUPLICATED_USER)
                except Exception as e:
                    self.logger.error('uid:{}\t{}'.format(uid, e))
                    pass

            if request.user.is_authenticated():
                logout(request)

    def process_response(self, request, response):
        """
        Write cookies to implement single sign on with reply
        """
        if settings.ACOMEA_SSO_SERVER and hasattr(request, 'user'):

            if re.match("%s|%s" % (settings.MEDIA_URL, settings.STATIC_URL), request.path):
                return response

            kw = {'domain': '.acomea.it'}
            if request.user.is_authenticated():
                response.set_cookie('uid', request.user.username, **kw)

                if hasattr(request.user, 'sso'):
                    response.set_cookie('token', request.user.sso.get('token'), **kw)
            else:
                response.delete_cookie('token', **kw)
                response.delete_cookie('uid', **kw)

        return response
