# coding: utf-8
import datetime
import logging
import os

from acomea.sso.models import SSOUserNotObfuscable
from django.conf import settings
from django.contrib.auth import models
from django.core.files.temp import NamedTemporaryFile
from django.core.mail import EmailMultiAlternatives
from django.db.models import Q

from codecs import open

from acomea.sso import models as sso_models
from acomea.sso.utils.sso_client import SSOClient


class MatchUsers(object):
    """
    Controlla che gli username degli utenti su acomea siano uguali a quelli su SSO a parità di email.
    In caso di differenza gli utenti acomea (con rac != 6 o 9) saranno offuscati perchè superati.
    In coda verrà mandato report via email

    option_list = BaseCommand.option_list + (
        make_option('-b', '--begin', type='int', dest='begin', help='Begin from index'),
        make_option('-s', '--limit', type='int', dest='limit', help='Limit list retrieved'),
        make_option('-u', '--uid', dest='uid', help='Uid to manage', ),
        make_option('-m', '--email', dest='email', help='Email to manage', ),
        make_option('-f', '--file', dest='file', default=None, help="Import list uid from file"),

        make_option('-c', '--console', action='store_true', default=False, help='Debug - write logging to console'),
        make_option('-l', '--debug-level', default='info', help='Set debug level (debug, info, warnings) for console'),
        make_option('-x', '--no-stop', action='store_true', default=False, help="Don't stop on error, just log"),
        make_option('-n', '--dry-run', action='store_true', default=False, help="Only report, no obfuscate"),
    )
    """
    logger = logging.getLogger('acomea.sso.match.obfuscate')

    users = list()
    errors = dict()
    obfuscated = list()
    not_obfuscated = list()
    matched = list()
    rac_not_allowed = list()
    user_updated = list()
    with_spaces = list()

    def do(self, options):
        self.match(options)
        self.check_email_with_space()
        self.send_report_by_email()

    def match(self, options):
        begin = options.get('begin')
        limit = options.get('limit')
        uid = options.get('uid')
        email = options.get('email')
        file_input = options.get('file')

        self.users = models.User.objects.filter(is_active=True)
        if uid:
            self.users = self.users.filter(username=uid)

        if email:
            self.users = self.users.filter(email=email)

        if file_input:
            with open(file_input, "rb") as input:
                list_to_filter = [(uid.split(";")[2]) for uid in input]
                if limit:
                    list_to_filter = list_to_filter[:limit]
                self.users = self.users.filter(username__in=list_to_filter)
        try:
            sso_client = SSOClient()
            self.logger.info('Start match users on sso')

            index_begin = 0
            if begin:
                self.users = self.users[begin:]
                index_begin += begin
            if limit:
                self.users = self.users[:limit]

            count = len(self.users)
            for index, user in enumerate(self.users):
                self.logger.debug(u"Try {}...({})\n".format(user.email, count))
                count -= 1
                try:
                    # start from email
                    sso_detail = sso_client.detail(email=user.email).get('data')
                    if not user.username == sso_detail.get('uid'):
                        if self.check_rac_status_not_allowed(user=user, sso_detail=sso_detail):
                            continue

                        self.obfuscate(user=user, sso_detail=sso_detail, options=options)
                        continue

                    self.set_matched(user=user, sso_detail=sso_detail)

                except Exception as e:
                    try:
                        # start from username
                        sso_detail = sso_client.detail(user.username).get('data')
                        if sso_detail.get('disabled') == '1':
                            if self.check_rac_status_not_allowed(user=user, sso_detail=sso_detail):
                                continue

                            self.obfuscate(user=user, sso_detail=sso_detail, options=options)
                            continue

                        self.update_user(user=user, sso_detail=sso_detail, options=options)
                    except Exception as e1:
                        if self.check_rac_status_not_allowed(user=user, sso_detail=None):
                            continue

                        self.obfuscate(user=user, sso_detail=None, options=options)
                        # self.errors[user.email] = e
                        self.logger.error(u"User: {} {}\n".format(user.email, e))
                        if options.get('no_stop'):
                            pass
                        else:
                            raise

        except Exception as e:
            self.logger.fatal(e)
            raise
        self.logger.info(u"Users checked: {}".format(len(self.users)))
        self.logger.info(u"Users matched: {}".format(len(self.matched)))
        self.logger.info(u"Users errors: {}".format(len(self.errors)))
        self.logger.info(u"Users obfuscated: {}".format(len(self.obfuscated)))
        self.logger.info(u"Users ar|g5=C not obfuscated: {}".format(len(self.not_obfuscated)))
        self.logger.info(u"Users updated: {}".format(len(self.user_updated)))
        self.logger.info(u"Users not match with rac in 6|9 : {}".format(len(self.rac_not_allowed)))

    def obfuscate(self, user, sso_detail, options):
        record = {
            'user': user,
            'sso_user': sso_detail
        }
        if not options.get('dry_run'):
            try:
                user.obfuscate()
                self.obfuscated.append(record)
                self.logger.info(u"{} OBFUSCATED".format(user.email))
            except SSOUserNotObfuscable:
                self.not_obfuscated.append(record)
                self.logger.info(u"{} ar|g5=C NON OBFUSCATED".format(user.email))
        else:
            if sso_detail.get('ar') == 'C' or sso_detail.get('g5') == 'C':
                self.not_obfuscated.append(record)
                self.logger.info(u"{} ar|g5=C NON OBFUSCATED".format(user.email))
            else:
                self.obfuscated.append(record)
                self.logger.info(u"{} OBFUSCATED".format(user.email))

    def update_user(self, user, sso_detail, options):
        if not options.get('dry_run'):
            sso_models.update_dj_user(user, sso_detail, None)
        record = {
            'user': user,
            'sso_user': sso_detail
        }
        self.user_updated.append(record)
        self.logger.info(u"{} UPDATED EMAIL".format(user.email))

    def set_matched(self, user, sso_detail):
        record = {
            'user': user,
            'sso_user': sso_detail
        }
        self.matched.append(record)
        self.logger.debug(u"{} EQUAL".format(user.email))

    def check_rac_status_not_allowed(self, user, sso_detail):
        if hasattr(user, 'profilo') and hasattr(user.profilo, 'rac') and user.profilo.rac.stato in ['6', '9']:
            record = {
                'user': user,
                'sso_user': sso_detail
            }
            self.rac_not_allowed.append(record)
            self.logger.info(u"{} NOT MATCH, RAC 6|9".format(user.email))
            return True

    def check_email_with_space(self):
        users = models.User.objects.filter(Q(email__endswith=' ') | Q(email__startswith=' '))
        self.with_spaces = [user for user in users]
        self.logger.info(u"Users with_spaces: {}".format(len(self.with_spaces)))

    def generate_report(self):
        """
        generate report about dj_user vs SSO and how many obfuscated
        :return:
        """
        with NamedTemporaryFile('wb', delete=False) as f:
            output = f.name
            f.write(';email;username;date joined;last login;rac;\n')
            for item in self.rac_not_allowed:
                user = item.get('user')
                f.write('RAC 6|9;{};{};{};{};{}\n'.format(
                    user.email, user.username, user.date_joined, user.last_login, user.profilo.rac.stato
                ))
            for user in self.with_spaces:
                stato_rac = user.profilo.rac.stato if hasattr(user, 'profilo') and hasattr(user.profilo, 'rac') else ''
                f.write('SPACES IN EMAIL;{};{};{};{};{}\n'.format(
                    user.email, user.username, user.date_joined, user.last_login, stato_rac
                ))
            for item in self.obfuscated:
                user = item.get('user')
                stato_rac = user.profilo.rac.stato if hasattr(user, 'profilo') and hasattr(user.profilo, 'rac') else ''
                f.write('OBFUSCATED;{};{};{};{};{}\n'.format(
                    user.email, user.username, user.date_joined, user.last_login, stato_rac
                ))
            for item in self.not_obfuscated:
                user = item.get('user')
                stato_rac = user.profilo.rac.stato if hasattr(user, 'profilo') and hasattr(user.profilo, 'rac') else ''
                f.write('NOT OBFUSCATED, ar|g5=C;{};{};{};{};{}\n'.format(
                    user.email, user.username, user.date_joined, user.last_login, stato_rac
                ))
            for item in self.user_updated:
                user = item.get('user')
                stato_rac = user.profilo.rac.stato if hasattr(user, 'profilo') and hasattr(user.profilo, 'rac') else ''
                f.write('USER UPDATED;{};{};{};{};{}\n'.format(
                    user.email, user.username, user.date_joined, user.last_login, stato_rac
                ))
            for key, value in self.errors.items():
                f.write('ERROR;{};{}\n'.format(key, value))

        result = open(output, 'rb').read()

        if output and os.path.exists(output):
            os.unlink(output)
        return result

    def send_report_by_email(self, to_email=None):
        """
        send report about dj_user vs SSO and how many obfuscated
        :param to_email: force address email to
        :return:
        """
        from_email = settings.ACOMEA_SSO_NO_REPLY_EMAIL

        if not to_email:
            to_email = getattr(settings, 'DESTINATARIO_EMAIL', settings.ACOMEA_SSO_MATCH_USER_DJ_VS_SSO_EMAIL_TO)

        import_result = """
            \n\tTOTAL RESULT:
                - utenti controllati:..........................{users}
                - utenti corrispondenti:.......................{matched}
                - utenti in errore:............................{errors}
                - utenti offuscati:............................{obfuscated}
                - utenti ar|g5=C NON offuscati:................{not_obfuscated}
                - utenti aggiornati:...........................{user_updated}
                - utenti non corrispondenti, ma con rac 6|9 :..{rac_not_allowed}
                - utenti con spazi nell'email :................{with_spaces}
    
            """
        import_result = import_result.format(
            users=len(self.users),
            matched=len(self.matched),
            errors=len(self.errors),
            obfuscated=len(self.obfuscated),
            not_obfuscated=len(self.not_obfuscated),
            user_updated=len(self.user_updated),
            rac_not_allowed=len(self.rac_not_allowed),
            with_spaces=len(self.with_spaces)
        )

        csv = self.generate_report()

        with NamedTemporaryFile('wb', delete=False) as f:
            f.write(csv)
            mail_message = EmailMultiAlternatives(
                subject='Acomea check users DJ vs SSO {}'.format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
                body='{}'.format(import_result),
                to=to_email,
                from_email=from_email
            )
            mail_message.attach('report.csv', csv, 'text/csv')
            mail_message.send()
